﻿using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.Server;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Net;
using System.Web;
using System.Xml;

namespace TicketM.Models
{
    public class Tickets
    {
        ArrayList assigneeList;
        ArrayList efforts;
        int ctr;
        public DataTable GetTickets(string sProject)
        {
            DataTable TicketsTable = new DataTable();
            DataColumn PkTicketIdColumn = TicketsTable.Columns.Add("TicketId", typeof(System.Int32));
            TicketsTable.Columns.Add("TicketTitle", typeof(string));
            try
            {
                #region Get Variable Values from existing config file
                string uName;
                string passwd;
                string domain;
                string tfs;
                string collection;
                string _project;
                SetGenVarValues(sProject, out uName, out passwd, out domain, out tfs, out collection, out _project);
                #endregion

                TfsTeamProjectCollection tpc = Connect2Tfs(uName, passwd, domain, tfs, collection);
                WorkItemStore wiStore = tpc.GetService<WorkItemStore>();
                WorkItemCollection wiColl = wiStore.Query("SELECT [System.Id], [System.Title] FROM WorkItems WHERE [System.TeamProject] = '" + _project + "'  AND  [System.WorkItemType] = 'Ticket'");
                foreach (WorkItem wi in wiColl)
                {
                    DataRow TicketRow = TicketsTable.NewRow();
                    TicketRow["TicketId"] = wi.Id;
                    TicketRow["TicketTitle"] = wi.Title;
                    TicketsTable.Rows.Add(TicketRow);
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return TicketsTable;
        }

        private static TfsTeamProjectCollection Connect2Tfs(string uName, string passwd, string domain, string tfs, string collection)
        {
            NetworkCredential crednetial = new NetworkCredential(uName, passwd, domain);
            TfsTeamProjectCollection tpc;
            string _uri = "http://" + tfs + ":8080/tfs/" + collection;
            try
            {
                tpc = new TfsTeamProjectCollection(new Uri(_uri), crednetial);
            }
            catch (Exception ex)
            {

                throw ex;
            }
            return tpc;
        }

        public DataTable GetTicket(int id, string sProject)
        {
            DataTable TicketTable = new DataTable();
            TicketTable.Columns.Add("ID", typeof(System.Int32));
            TicketTable.Columns.Add("State", typeof(string));
            TicketTable.Columns.Add("Assigned To", typeof(string));
            TicketTable.Columns.Add("Description", typeof(string));

            try
            {
                #region Get Variable Values from existing config file
                string uName;
                string passwd;
                string domain;
                string tfs;
                string collection;
                string _project;
                SetGenVarValues(sProject, out uName, out passwd, out domain, out tfs, out collection, out _project);
                #endregion

                TfsTeamProjectCollection tpc = Connect2Tfs(uName, passwd, domain, tfs, collection);
                WorkItemStore wiStore = tpc.GetService<WorkItemStore>();
                WorkItem wi = wiStore.GetWorkItem(id);

                DataRow fldRow1 = TicketTable.NewRow();
                fldRow1["ID"] = wi.Id;
                fldRow1["State"] = wi.State;
                fldRow1["Assigned To"] = wi.Fields["System.AssignedTo"].Value.ToString();
                fldRow1["Description"] = wi.Description;
                TicketTable.Rows.Add(fldRow1);


            }
            catch (Exception ex)
            {
                throw ex;
            }
            return TicketTable;

        }

        public DataTable GetEfforts(int id, string sProject)
        {

            DataTable EffortsTable = new DataTable();
            EffortsTable.Columns.Add("Name", typeof(string));
            EffortsTable.Columns.Add("Efforts", typeof(System.Int32));
            try
            {
                #region Get Variable Values from existing config file
                string uName;
                string passwd;
                string domain;
                string tfs;
                string collection;
                string _project;
                SetGenVarValues(sProject, out uName, out passwd, out domain, out tfs, out collection, out _project);
                #endregion

                TfsTeamProjectCollection tpc = Connect2Tfs(uName, passwd, domain, tfs, collection);
                WorkItemStore wiStore = tpc.GetService<WorkItemStore>();
                WorkItem wi = wiStore.GetWorkItem(id);
                RevisionCollection revColl = wi.Revisions;
                Revision LastRev = null;
                foreach (Revision rev in revColl)
                {
                    LastRev = rev;
                }

                assigneeList = new ArrayList();
                efforts = new ArrayList();
                foreach (Revision rev in revColl)
                {
                    if (rev.Fields["System.AssignedTo"].Value.ToString() != rev.Fields["System.AssignedTo"].OriginalValue.ToString() && rev.Fields["System.AssignedTo"].OriginalValue.ToString() != "")
                    {

                        if (IsAssigneePresent(rev.Fields["System.AssignedTo"].OriginalValue.ToString()))
                        {
                            int eff = 0;
                            eff = Int32.Parse(efforts[ctr].ToString());
                            eff += Int32.Parse(rev.Fields["SSGS.MyEfforts"].Value.ToString());
                            efforts[ctr] = eff;
                        }
                        else
                        {
                            assigneeList.Add(rev.Fields["System.AssignedTo"].OriginalValue.ToString());
                            efforts.Add(Int32.Parse(rev.Fields["SSGS.MyEfforts"].Value.ToString()));
                        }
                    }
                }
                for (int i = 0; i < assigneeList.Count; i++)
                {
                    DataRow AssigneeRow = EffortsTable.NewRow();
                    AssigneeRow["Name"] = assigneeList[i].ToString();
                    AssigneeRow["Efforts"] = efforts[i].ToString();
                    EffortsTable.Rows.Add(AssigneeRow);
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return EffortsTable;
        }
        private bool IsAssigneePresent(string assignee)
        {
            bool assigneePresent = false;
            ctr = 0;
            if (assigneeList != null)
            {
                foreach (string candidate in assigneeList)
                {
                    if (candidate == assignee)
                    {
                        assigneePresent = true;
                        break;
                    }
                    ctr++;
                }
            }
            return assigneePresent;
        }
        /// <summary>
        /// This method reads the areas structure for a selected project
        /// </summary>
        /// <param name="sProject">Session variable that represents selected project</param>
        /// <returns></returns>
        public XmlDocument GetAreasStructure(string sProject)
        {
            #region Get Variable Values from existing config file
            string uName;
            string passwd;
            string domain;
            string tfs;
            string collection;
            string _project;
            SetGenVarValues(sProject, out uName, out passwd, out domain, out tfs, out collection, out _project);
            #endregion

            try
            {
                TfsTeamProjectCollection tpc = Connect2Tfs(uName, passwd, domain, tfs, collection);
                ICommonStructureService StructureService = (ICommonStructureService)tpc.GetService(typeof(ICommonStructureService));
                //Gets Area/Iteration base Project

                ProjectInfo projectInfo = StructureService.GetProjectFromName(_project);
                NodeInfo[] nodes = StructureService.ListStructures(projectInfo.Uri);
                XmlDocument doc1 = new XmlDocument();
                doc1.LoadXml(StructureService.GetNodesXml(new string[] { nodes[0].Uri }, true).InnerXml);
                XmlNodeList allNodes = doc1.FirstChild.SelectNodes("//*");
                return ProcessNode(allNodes, sProject);
            }
            catch (Exception ex)
            {
                throw ex;
            }

        }
        /// <summary>
        /// This is a helper method that converts the passed nodes (areas) into a flat XmlDocument object
        /// </summary>
        /// <param name="allNodes">All area structure nodes for the selected project</param>
        /// <param name="sProject">Session variable representing selected project</param>
        /// <returns></returns>
        private XmlDocument ProcessNode(XmlNodeList allNodes, string sProject)
        {
            XmlDocument doc = new XmlDocument();
            XmlDeclaration dec = doc.CreateXmlDeclaration("1.0", null, null);
            doc.AppendChild(dec);
            XmlElement root1 = doc.CreateElement("Project");
            XmlAttribute nameAttr = doc.CreateAttribute("Name");
            nameAttr.Value = sProject;
            root1.Attributes.Append(nameAttr);
            doc.AppendChild(root1);
            try
            {
                for (int i = 1; i < allNodes.Count; i++)
                {

                    if (allNodes[i].Name != "Children")
                    {
                        XmlNode elem = doc.CreateNode(XmlNodeType.Element, "SubArea", null);
                        XmlAttribute attrib = doc.CreateAttribute("Name");
                        //Structure service contains format that is not compatible further. So we need to process it first.
                        attrib.Value = allNodes[i].Attributes["Path"].Value.Split(new char[] { '\\' }, 4).GetValue(3).ToString();
                        elem.Attributes.Append(attrib);
                        root1.AppendChild(elem);
                    }
                }
            }
            catch (Exception ex)
            {

                throw ex;
            }
            return doc;
        }
        /// <summary>
        /// This method reads the existing products from ticket work item type for a given project.
        /// The arraylist that is returened is bound to a datagrid on Products.aspx page
        /// </summary>
        /// <param name="sProject">Represents the session variable that contains selected project</param>
        /// <returns></returns>
        public ArrayList GetProductsList(string sProject)
        {
            ArrayList Products = new ArrayList();
            try
            {
                #region Get Variable Values from existing config file
                string uName;
                string passwd;
                string domain;
                string tfs;
                string collection;
                string _project;
                SetGenVarValues(sProject, out uName, out passwd, out domain, out tfs, out collection, out _project);
                #endregion

                TfsTeamProjectCollection tpc = Connect2Tfs(uName, passwd, domain, tfs, collection);
                var wiStore = tpc.GetService<WorkItemStore>();
                var project = wiStore.Projects[_project];
                WorkItemType wiType = project.WorkItemTypes["Ticket"];
                XmlDocument doc1 = wiType.Export(true);
                XmlNode root1 = doc1.DocumentElement;
                XmlNodeList nodes = root1.SelectNodes("//FIELDS/FIELD[@name='Product']/ALLOWEDVALUES/LISTITEM");
                foreach (XmlNode product in nodes)
                {
                    Products.Add(product.Attributes["value"].Value);
                }
            }
            catch (Exception ex)
            {

                throw ex;
            }
            return Products;
        }
        /// <summary>
        /// This method updates the Ticket work item type for the selected project by adding the new product in the products list
        /// </summary>
        /// <param name="newProduct">Name of the new product to add</param>
        /// <param name="sProject">Session variable that represents the selected project</param>
        /// <returns></returns>
        public string SetProductsList(string newProduct, string sProject)
        {
            string returnMsg;
            try
            {
                #region Get Variable Values from existing config file
                string uName;
                string passwd;
                string domain;
                string tfs;
                string collection;
                string _project;
                SetGenVarValues(sProject, out uName, out passwd, out domain, out tfs, out collection, out _project);
                #endregion

                //Connect to TFS and from the selected project get the XML of Ticket work item type definition
                TfsTeamProjectCollection tpc = Connect2Tfs(uName, passwd, domain, tfs, collection);
                var wiStore = tpc.GetService<WorkItemStore>();
                var project = wiStore.Projects[_project];
                WorkItemType wiType = project.WorkItemTypes["Ticket"];
                XmlDocument doc1 = wiType.Export(true);
                XmlNode root1 = doc1.DocumentElement;


                //Check if the product already exists
                XmlNode item = root1.SelectSingleNode("//FIELDS/FIELD[@name='Product']/ALLOWEDVALUES/LISTITEM[@value=\"" + newProduct + "\"]");
                if (item == null) //if the product does not exists in the Ticket work item of that project create a node for that product
                {
                    XmlNode node = root1.SelectSingleNode("//FIELDS/FIELD[@name='Product']/ALLOWEDVALUES");
                    XmlNode NewProductNode = doc1.CreateNode(XmlNodeType.Element, "LISTITEM", null);
                    XmlAttribute valueAttrib = doc1.CreateAttribute("value");
                    valueAttrib.Value = newProduct;
                    NewProductNode.Attributes.Append(valueAttrib);

                    //append it at appropriate place
                    node.AppendChild(NewProductNode);
                    //import the work item type back in the same project
                    project.WorkItemTypes.Import(doc1.DocumentElement);
                    returnMsg = "Product Added";
                }
                else
                {
                    returnMsg = "Same named product found";
                }
            }
            catch (Exception ex)
            {

                throw ex;
            }
            return returnMsg;
        }

        private static void SetGenVarValues(string sProject, out string uName, out string passwd, out string domain, out string tfs, out string collection, out string _project)
        {
            try
            {
                XmlDocument doc = new XmlDocument();
                //doc.Load(HttpContext.Current.Server.MapPath("/Configuration/TicketProcessingService.exe.config"));
                doc.Load(HttpContext.Current.Server.MapPath("/TicketM/Configuration/TicketProcessingService.exe.config"));
                uName = doc.DocumentElement.SelectSingleNode("/configuration/appSettings/add[@key='UserName']").Attributes["value"].Value;
                passwd = doc.DocumentElement.SelectSingleNode("/configuration/appSettings/add[@key='Password']").Attributes["value"].Value;
                domain = doc.DocumentElement.SelectSingleNode("/configuration/appSettings/add[@key='Domain']").Attributes["value"].Value;
                tfs = doc.DocumentElement.SelectSingleNode("/configuration/appSettings/add[@key='Tfs']").Attributes["value"].Value;
                collection = doc.DocumentElement.SelectSingleNode("/configuration/appSettings/add[@key='Collection']").Attributes["value"].Value;
                _project = sProject;
            }
            catch (Exception ex)
            {

                throw ex;
            }
        }

        /// <summary>
        /// This is a method in which the assignment constraints as set in ticket work item for selected project are read and stored in the AreasStruct.xml file
        /// </summary>
        /// <param name="docParam">This parameter is the current area structure read in GetAreaStructure method is passed on</param>
        /// <param name="sProject"></param>
        public void GetRoutingConditions(XmlDocument docParam, string sProject)
        {
            try
            {
                //The parameter docParam is the Area Structure of the selected project.
                #region Get Variable Values from existing config file
                string uName;
                string passwd;
                string domain;
                string tfs;
                string collection;
                string _project;
                SetGenVarValues(sProject, out uName, out passwd, out domain, out tfs, out collection, out _project);
                #endregion
                //Connect to TFS
                TfsTeamProjectCollection tpc = Connect2Tfs(uName, passwd, domain, tfs, collection);
                var wiStore = tpc.GetService<WorkItemStore>();
                var project = wiStore.Projects[_project];
                //Get the existing definition of Ticket WITD
                WorkItemType wiType = project.WorkItemTypes["Ticket"];
                //Export the Ticket WITD in a XML Document structure
                XmlDocument doc1 = wiType.Export(true);
                XmlNode root1 = doc1.DocumentElement;
                //Get the node for 'Assgined To' Field
                XmlNode node = root1.SelectSingleNode("//FIELDS/FIELD[@name='Assigned To']");


                int ctr = 0;
                //Get all nodes for Area from AreasStruct.xml file
                XmlNodeList areaNodes = docParam.FirstChild.NextSibling.SelectNodes("/*/*");
                //Read each area node    
                while (ctr < areaNodes.Count)
                {
                    string areaPath = areaNodes[ctr].Attributes["Name"].Value.ToString();
                    //Get Area Id for each area node        
                    int areaId = GetAreaId(areaPath, _project);
                    XmlNode defaultAssigneeNode = docParam.CreateNode(XmlNodeType.Element, "Assignee", null);
                    if (node.SelectSingleNode("WHEN[@value='" + areaId + "']/*/*") != null)
                    {
                        defaultAssigneeNode.InnerText = node.SelectSingleNode("WHEN[@value='" + areaId + "']/*/*").Attributes["value"].Value.ToString();

                        XmlNode root = docParam.DocumentElement;
                        XmlNode targetAreaNode = root.SelectSingleNode("/Project/SubArea[@Name='" + areaPath + "']");
                        targetAreaNode.AppendChild(defaultAssigneeNode);
                    }
                    ctr++;
                }
                docParam.Save(HttpContext.Current.Server.MapPath("/TicketM/App_Data/AreasStruct.xml"));
            }
            catch (Exception ex)
            {

                throw ex;
            }

        }
        /// <summary>
        /// This method reads existing routing data from the configuration file AreasStruct.xml and converts it to a datatable.
        /// That datatable is bound to the datagrid on the Routing.aspx
        /// </summary>
        /// <param name="sProject">Session variable that represents selected project</param>
        /// <returns></returns>
        public DataTable ReadRoutingFromTFS(string sProject)
        {
            DataTable RoutingTable = new DataTable("RoutingTable");
            RoutingTable.Columns.Add("AreaPath", typeof(string));
            RoutingTable.Columns.Add("AssignedTo", typeof(string));

            try
            {
                XmlDocument doc = new XmlDocument();
                doc.Load(HttpContext.Current.Server.MapPath(@"/TicketM/App_Data/AreasStruct.xml"));
                XmlNode ProjectNode = doc.DocumentElement.SelectSingleNode("/Project[@Name='" + sProject + "']");
                if (ProjectNode.HasChildNodes)
                {
                    XmlNodeList RoutingList = ProjectNode.ChildNodes;
                    foreach (XmlNode subarea in RoutingList)
                    {
                        DataRow dRow = RoutingTable.NewRow();
                        dRow["AreaPath"] = subarea.Attributes["Name"].Value.ToString();
                        if (subarea.HasChildNodes)
                            dRow["AssignedTo"] = subarea.FirstChild.InnerText;
                        else
                            dRow["AssignedTo"] = "";
                        RoutingTable.Rows.Add(dRow);
                    }
                }
            }
            catch (Exception ex)
            {

                throw ex;
            }
            return RoutingTable;
        }
        public ArrayList GetTeamMembers(string sProject)
        {
            ArrayList UsersAL = new ArrayList();
            try
            {
                #region Get Variable Values from existing config file
                string uName;
                string passwd;
                string domain;
                string tfs;
                string collection;
                string _project;
                SetGenVarValues(sProject, out uName, out passwd, out domain, out tfs, out collection, out _project);
                #endregion

                TfsTeamProjectCollection tpc = Connect2Tfs(uName, passwd, domain, tfs, collection);
                var wiStore = tpc.GetService<WorkItemStore>();
                var project = wiStore.Projects[_project];
                var gss = tpc.GetService<IGroupSecurityService>();
                Identity SIDs = gss.ReadIdentity(SearchFactor.AccountName, "[" + project.Name + "]\\Contributors", QueryMembership.Expanded);
                Identity[] Users = gss.ReadIdentities(SearchFactor.Sid, SIDs.Members, QueryMembership.Direct);
                foreach (Identity user in Users)
                {

                    if (user.SecurityGroup)
                    {
                        UsersAL.Add("[project]\\" + user.DisplayName);
                    }
                    else
                    {
                        UsersAL.Add(user.DisplayName);
                    }

                }
            }
            catch (Exception ex)
            {

                throw ex;
            }
            return UsersAL;
        }
        public ArrayList GetTeamMembersEmails(string sProject)
        {
            ArrayList UsersEmailsAL = new ArrayList();
            try
            {
                #region Get Variable Values from existing config file
                string uName;
                string passwd;
                string domain;
                string tfs;
                string collection;
                string _project;
                SetGenVarValues(sProject, out uName, out passwd, out domain, out tfs, out collection, out _project);
                #endregion

                TfsTeamProjectCollection tpc = Connect2Tfs(uName, passwd, domain, tfs, collection);
                var wiStore = tpc.GetService<WorkItemStore>();
                var project = wiStore.Projects[_project];
                var gss = tpc.GetService<IGroupSecurityService>();
                Identity SIDs = gss.ReadIdentity(SearchFactor.AccountName, "[" + project.Name + "]\\Contributors", QueryMembership.Expanded);
                Identity[] Users = gss.ReadIdentities(SearchFactor.Sid, SIDs.Members, QueryMembership.Direct);
                foreach (Identity user in Users)
                {

                    if (!user.SecurityGroup)
                    {
                        UsersEmailsAL.Add(user.MailAddress);
                    }
                }
            }
            catch (Exception ex)
            {

                throw ex;
            }
            return UsersEmailsAL;
        }
        public int GetAreaId(string AreaPath, string sProject)
        {
            try
            {
                #region Get Variable Values from existing config file
                string uName;
                string passwd;
                string domain;
                string tfs;
                string collection;
                string _project;
                SetGenVarValues(sProject, out uName, out passwd, out domain, out tfs, out collection, out _project);
                #endregion

                TfsTeamProjectCollection tpc = Connect2Tfs(uName, passwd, domain, tfs, collection);
                var wiStore = tpc.GetService<WorkItemStore>();
                var project = wiStore.Projects[_project];
                NodeCollection nodes = project.AreaRootNodes;

                Node area = project.FindNodeInSubTree(AreaPath, Node.TreeType.Area);
                return area.Id;

            }
            catch (Exception ex)
            {

                throw ex;
            }

        }

        /// <summary>
        /// This method synchornizes the escalation structure after changes are made
        /// </summary>
        /// <param name="doc">This represents the current area structure of the selected project</param>
        /// <param name="sProject">Session variable that represents the selected project</param>
        public void SyncEscalationStructure(XmlDocument doc, string sProject)
        {
            try
            {
                XmlDocument EscalationArea = new XmlDocument();
                EscalationArea.Load(HttpContext.Current.Server.MapPath("/TicketM/App_Data/AreaNoDaysEmails.xml")); //Existing escalation accounts
                //Read existing nodes in the Areas of the project (passed as parameter by calling method)
                XmlNodeList AllAreas = doc.DocumentElement.SelectNodes("//SubArea");
                foreach (XmlNode area in AllAreas)
                {
                    //check if a node in AreaNoDaysEmails.xml config file exists for that area
                    if (EscalationArea.SelectSingleNode("/Area_Email_Days/Area[@Project='" + sProject + "' and @Name='" + area.Attributes["Name"].Value + "']") == null) //if project node is present but a specific area does not exist
                    {
                        //if that node does not exist, create one
                        XmlNode root = EscalationArea.DocumentElement;
                        XmlNode NewArea = EscalationArea.CreateNode(XmlNodeType.Element, "Area", null);
                        XmlAttribute NewAreaName = EscalationArea.CreateAttribute("Name");
                        XmlAttribute NewEmail = EscalationArea.CreateAttribute("Email");
                        XmlAttribute NewDays = EscalationArea.CreateAttribute("Days");
                        XmlAttribute NewProject = EscalationArea.CreateAttribute("Project");
                        //set standard values to attributes and area name as of selected area
                        NewEmail.Value = "";
                        NewDays.Value = "2";
                        NewAreaName.Value = area.Attributes["Name"].Value;
                        NewProject.Value = sProject;

                        NewArea.Attributes.Append(NewAreaName);
                        NewArea.Attributes.Append(NewDays);
                        NewArea.Attributes.Append(NewEmail);
                        NewArea.Attributes.Append(NewProject);
                        //append the node to the structure and save the file
                        root.AppendChild(NewArea);
                        EscalationArea.Save(HttpContext.Current.Server.MapPath("/TicketM/App_Data/AreaNoDaysEmails.xml"));
                    }
                }
            }
            catch (Exception ex)
            {

                throw ex;
            }
        }

        public ArrayList GetProjects()
        {
            ArrayList projects = new ArrayList();
            string uName;
            string passwd;
            string domain;
            string tfs;
            string collection;
            string _project;
            try
            {
                SetGenVarValues("", out uName, out passwd, out domain, out tfs, out collection, out _project);
                TfsTeamProjectCollection tpc = Connect2Tfs(uName, passwd, domain, tfs, collection);
                WorkItemStore store = (WorkItemStore)tpc.GetService(typeof(WorkItemStore));
                foreach (Project project in store.Projects)
                {
                    projects.Add(project.Name);
                }
            }
            catch (Exception ex)
            {

                throw ex;
            }
            return projects;

        }

        public bool AddTicketWIT2Project(string sProject)
        {
            bool retval = false;
            #region Get Variable Values from existing config file
            string uName;
            string passwd;
            string domain;
            string tfs;
            string collection;
            string _project;
            SetGenVarValues(sProject, out uName, out passwd, out domain, out tfs, out collection, out _project);
            #endregion
            try
            {
                TfsTeamProjectCollection tpc = Connect2Tfs(uName, passwd, domain, tfs, collection);
                var wiStore = tpc.GetService<WorkItemStore>();
                var project = wiStore.Projects[_project];
                XmlDocument doc1 = new XmlDocument();
                //doc1.Load(HttpContext.Current.Server.MapPath("/App_Data/Ticket.xml"));
                doc1.Load(HttpContext.Current.Server.MapPath("/TicketM/App_Data/Ticket.xml"));
                project.WorkItemTypes.Import(doc1.DocumentElement);
                XmlDocument doc2 = new XmlDocument();
                doc2.Load(HttpContext.Current.Server.MapPath("/TicketM/App_Data/Projects.xml"));
                XmlNode root = doc2.DocumentElement;
                XmlNode node = root.SelectSingleNode("//projects");
                XmlNode NewProductNode = doc2.CreateNode(XmlNodeType.Element, "project", null);
                XmlAttribute nameAttrib = doc2.CreateAttribute("name");
                nameAttrib.Value = _project;
                NewProductNode.Attributes.Append(nameAttrib);

                //append it at appropriate place
                node.AppendChild(NewProductNode);
                doc2.Save(HttpContext.Current.Server.MapPath("/TicketM/App_Data/Projects.xml"));
                                
                XmlDocument doc3 = new XmlDocument();
                doc3.Load(HttpContext.Current.Server.MapPath("/TicketM/App_Data/AreaNoDaysEmails.xml"));
                XmlNode root3 = doc3.DocumentElement;
                XmlNode node3 = root3.SelectSingleNode("//Area_Email_Days");
                XmlNode NewAreaNode = doc3.CreateNode(XmlNodeType.Element, "Area", null);
                XmlAttribute nameAttrib1 = doc3.CreateAttribute("Name");
                nameAttrib1.Value = _project;
                XmlAttribute nameAttrib2 = doc3.CreateAttribute("Days");
                nameAttrib2.Value = "2";
                XmlAttribute nameAttrib3 = doc3.CreateAttribute("Email");
                nameAttrib3.Value = "";
                XmlAttribute nameAttrib4 = doc3.CreateAttribute("Project");
                nameAttrib4.Value = _project;
                NewAreaNode.Attributes.Append(nameAttrib1);
                NewAreaNode.Attributes.Append(nameAttrib2);
                NewAreaNode.Attributes.Append(nameAttrib3);
                NewAreaNode.Attributes.Append(nameAttrib4);
                //append it at appropriate place
                node3.AppendChild(NewAreaNode);
                doc3.Save(HttpContext.Current.Server.MapPath("/TicketM/App_Data/AreaNoDaysEmails.xml"));

		retval = true;
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return retval;
        }

        public ArrayList ProjectsWithTicketing()
        {
            ArrayList projects = new ArrayList();
            XmlDocument doc = new XmlDocument();
            try
            {
                //doc.Load(HttpContext.Current.Server.MapPath("/App_Data/Projects.xml"));
                
                doc.Load(HttpContext.Current.Server.MapPath("/TicketM/App_Data/Projects.xml"));
                XmlNodeList projectNodes = doc.DocumentElement.SelectNodes("/projects/project");
                foreach (XmlNode projectNode in projectNodes)
                {
                    projects.Add(projectNode.Attributes["name"].Value);
                }
            }
            catch (Exception ex)
            {

                throw ex;
            }
            return projects;

        }
    }

}