﻿using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Xml;
using System.Collections;
using System.Collections.Specialized;

namespace TicketM.Configuration
{
    public partial class Routing : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (Session["sProject"].ToString() == "" || Session["sProject"] == null)
            {
                Session.Add("refPage", HttpContext.Current.Request.Url);
                Response.Redirect("ProjectSelection.aspx");
            }
            lblProject.Text = Session["sProject"].ToString();
            if (!Page.IsPostBack)
            {
                TicketM.Models.Tickets areas = new Models.Tickets();
                areas.GetRoutingConditions(areas.GetAreasStructure(Session["sProject"].ToString()), Session["sProject"].ToString());
            }
        }

        protected void GridView1_SelectedIndexChanged(object sender, EventArgs e)
        {
            GridViewRow selectedRow = GridView1.SelectedRow;
           AreaPath.Text = selectedRow.Cells[1].Text;
           AssignedTo.SelectedItem.Text = selectedRow.Cells[2].Text;
           StoreButton.Enabled = true;
        }

        protected void StoreButton_Click(object sender, EventArgs e)
        {
            XmlDocument doc = new XmlDocument();
            doc.Load(HttpContext.Current.Server.MapPath("~\\TicketM\\App_Data\\AreasStruct.xml"));
            XmlNode root = doc.DocumentElement;
            XmlNode node = root.SelectSingleNode("SubArea[@Name='" + AreaPath.Text + "']");
            if (node.HasChildNodes)
            {
                node.FirstChild.InnerText = AssignedTo.SelectedItem.Text;
                doc.Save(HttpContext.Current.Server.MapPath("~\\TicketM\\App_Data\\AreasStruct.xml"));
                StoreButton.Enabled = false;
            }
            else
            {
                XmlNode userNode = doc.CreateNode(XmlNodeType.Element, "Assignee", null);
                userNode.InnerText = AssignedTo.SelectedItem.Text;
                node.AppendChild(userNode);
                doc.Save(HttpContext.Current.Server.MapPath("~\\TicketM\\App_Data\\AreasStruct.xml"));
                StoreButton.Enabled = false;
            }
            GridView1.DataBind();
            AssignedTo.DataBind();
            UpdateButton.Enabled = true;
        }

        protected void UpdateButton_Click(object sender, EventArgs e)
        {
            #region Get Variable Values from existing config file
            XmlDocument doc = new XmlDocument();
            doc.Load(HttpContext.Current.Server.MapPath("/TicketM/Configuration/TicketProcessingService.exe.config"));
            string uName = doc.DocumentElement.SelectSingleNode("/configuration/appSettings/add[@key='UserName']").Attributes["value"].Value;
            string passwd = doc.DocumentElement.SelectSingleNode("/configuration/appSettings/add[@key='Password']").Attributes["value"].Value;
            string domain = doc.DocumentElement.SelectSingleNode("/configuration/appSettings/add[@key='Domain']").Attributes["value"].Value;
            string tfs = doc.DocumentElement.SelectSingleNode("/configuration/appSettings/add[@key='Tfs']").Attributes["value"].Value;
            string collection = doc.DocumentElement.SelectSingleNode("/configuration/appSettings/add[@key='Collection']").Attributes["value"].Value;
            string _project = Session["sProject"].ToString();
            #endregion

            #region Connect to TFS, get project name and Ticket Type
            NetworkCredential crednetial = new NetworkCredential(uName,passwd,domain);
            string _uri = "http://" + tfs + ":8080/tfs/" + collection;
            TfsTeamProjectCollection tpc = new TfsTeamProjectCollection(new Uri(_uri), crednetial);
            var wiStore = tpc.GetService<WorkItemStore>();
            var project = wiStore.Projects[_project];
            WorkItemType wiType = project.WorkItemTypes["Ticket"];
            #endregion

            #region Read Area Structure from a local file and get AreaIds for each AreaPath
            XmlDocument tempDoc = new XmlDocument();
            tempDoc.Load(HttpContext.Current.Server.MapPath("~\\TicketM\\App_Data\\AreasStruct.xml"));
            XmlNode ProjectNode = tempDoc.DocumentElement.SelectSingleNode("/Project[@Name='" + _project + "']");


            XmlNodeList AllAreaPathNodes = ProjectNode.ChildNodes;
            TicketM.Models.Tickets ticket = new Models.Tickets();
            List<int> areaIds = new List<int>();
            foreach (XmlNode areaPath in AllAreaPathNodes)
            {
                areaIds.Add(ticket.GetAreaId(areaPath.Attributes["Name"].Value.ToString(), Session["sProject"].ToString()));
            }
#endregion

            #region Read Ticket type definition and get AssignedTo field with contraints
            XmlDocument doc1 = wiType.Export(true);
            XmlNode root1 = doc1.DocumentElement;
            XmlNode AssignedToNode = root1.SelectSingleNode("//FIELDS/FIELD[@name='Assigned To']");
            #endregion
            
            
            int ctr = 0;
            while (ctr < areaIds.Count)
            {
                XmlNode AssigneeNode = AssignedToNode.SelectSingleNode("WHEN[@value='" + areaIds[ctr] + "']/*/*");
                if (AssigneeNode != null)
                {
                    AssigneeNode.Attributes["value"].Value = AllAreaPathNodes[ctr].FirstChild.InnerText;
                    ctr++;
                }
                else
                {
                    XmlNode WhenNode = doc1.CreateNode(XmlNodeType.Element, "WHEN", null);
                    XmlAttribute fieldAttrib = doc1.CreateAttribute("field");
                    fieldAttrib.Value = "System.AreaId";
                    XmlAttribute valueAttrib = doc1.CreateAttribute("value");
                    valueAttrib.Value = areaIds[ctr].ToString();
                    WhenNode.Attributes.Append(fieldAttrib);
                    WhenNode.Attributes.Append(valueAttrib);

                    XmlNode AllowedValuesNode = doc1.CreateNode(XmlNodeType.Element, "ALLOWEDVALUES", null);
                    XmlAttribute expandAttrib = doc1.CreateAttribute("expanditems");
                    expandAttrib.Value = "true";
                    AllowedValuesNode.Attributes.Append(expandAttrib);

                    XmlNode ListItemNode = doc1.CreateNode(XmlNodeType.Element, "LISTITEM", null);
                    XmlAttribute valueAttrib2 = doc1.CreateAttribute("value");
                    valueAttrib2.Value = AllAreaPathNodes[ctr].FirstChild.InnerText;
                    ListItemNode.Attributes.Append(valueAttrib2);

                    AllowedValuesNode.AppendChild(ListItemNode);
                    WhenNode.AppendChild(AllowedValuesNode);
                    AssignedToNode.AppendChild(WhenNode);
                    ctr++;
                }
            }
            project.WorkItemTypes.Import(doc1.InnerXml);
            UpdateButton.Enabled = false;
        }
    }
}