﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Xml;
using TicketM.ReportService;

namespace TicketM
{
    public partial class _Default : Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack)
            {
            
                GetProjectsTicketedNonTicketed();
            }
        }

        private void GetProjectsTicketedNonTicketed()
        {
            Models.Tickets ticketing = new Models.Tickets();
            ArrayList projects = null;
            try
            {
                projects = ticketing.GetProjects();
            }
            catch (Exception ex)
            {
                Logger.CreateLogEntry("!!ERROR in Default.aspx.cs while getting the project list: " + ex.Message);
                Response.Redirect("/CommonErrorPage.aspx");
            }

            XmlDocument doc = new XmlDocument();
            //doc.Load(Server.MapPath(@"\App_Data\Projects.xml"));
            doc.Load(Server.MapPath(@"\TicketM\App_Data\Projects.xml"));

            foreach (string project in projects)
            {
                if (doc.DocumentElement.SelectSingleNode("/projects/project[@name='" + project + "']") == null)
                {
                    this.NonTicketed.Items.Add(project);
                }
                else
                {
                    this.Ticketed.Items.Add(project);
                }
            }
        }

        protected void Save_Click(object sender, EventArgs e)
        {
            Models.Tickets ticketing = new Models.Tickets();

            try
            {
                if (ticketing.AddTicketWIT2Project(Session["project"].ToString()))
                {
                    NonTicketed.Items.Clear();
                    Ticketed.Items.Clear();
                    GetProjectsTicketedNonTicketed();
                    Save.Enabled = false;
                    addButton.Enabled = true;
                    //upload reports to the project
                    UploadReports(Session["project"].ToString());
                    Logger.CreateLogEntry("reports uploadted");
                    Logger.CreateLogEntry("** INFO Ticketing Support added to project " + Session["project"].ToString() + " at " + DateTime.Now.ToString());
                }
            }
            catch (Exception ex)
            {
                Logger.CreateLogEntry("!!ERROR in Default.aspx.cs while Adding Ticketing support to project: " + ex.Message);
            }
        }

        protected void addButton_Click(object sender, EventArgs e)
        {
            this.Ticketed.Items.Add(this.NonTicketed.SelectedItem);
            Session.Add("project", this.NonTicketed.SelectedItem.Text);
            this.NonTicketed.Items.Remove(this.NonTicketed.SelectedItem);
            addButton.Enabled = false;
            Save.Enabled = true;
        }
        protected void UploadReports(string teamproject)
        {
            //loop through a folder to get all rdl files
            XmlDocument doc = new XmlDocument();
            doc.Load(HttpContext.Current.Server.MapPath("/TicketM/Configuration/TicketProcessingService.exe.config"));
            XmlNode root = doc.DocumentElement;
            string reportservername = root.SelectSingleNode("/configuration/appSettings/add[@key='ReportServerName']").Attributes["value"].Value;
            string collection = root.SelectSingleNode("/configuration/appSettings/add[@key='Collection']").Attributes["value"].Value;
            string path = HttpContext.Current.Server.MapPath("/TicketM/Reports");
            //get credentials 
            string username = root.SelectSingleNode("/configuration/appSettings/add[@key='UserName']").Attributes["value"].Value;
            string password = root.SelectSingleNode("/configuration/appSettings/add[@key='Password']").Attributes["value"].Value;
            string domain = root.SelectSingleNode("/configuration/appSettings/add[@key='Domain']").Attributes["value"].Value;
            //@"C:\Users\administrator.SSGS\Documents\Reports";
            string[] files = Directory.GetFiles(path);
            foreach (string file in files)
            {
                //change the team projet name in the oroginal file
                string str = File.ReadAllText(file);
                string teamProject = "[" + teamproject + "]";
                str = str.Replace("[SSGS EMS]", teamProject);
                File.WriteAllText(file, str);
                // find only name of file 
                string[] stringSeparators = new string[] { "\\" };
                string[] result;
                result = file.Split(stringSeparators, StringSplitOptions.None);
                string reportName = result[result.Length - 1].Substring(0, result[result.Length - 1].Length - 4);
                ReportService.ReportingService2010 rs = new ReportService.ReportingService2010();

                //NetworkCredential credentials = new NetworkCredential("administrator", "P2ssw0rd", "SSGS");
                NetworkCredential credentials = new NetworkCredential(username, password, domain);
                rs.Credentials = credentials;
                rs.Url = "http://" + reportservername + "/reportserver/ReportService2010.asmx";
                //rs.Url = "http://ssgs-tfs2012/reportserver/ReportService2010.asmx";
                //find if the folder already exists
                //this team project tp is without square brackets

                ReportService.CatalogItem[] items = rs.ListChildren("/TfsReports/" + collection + "/" + teamproject, true);
                //localhost.CatalogItem[] items = rs.ListChildren("/TfsReports/DefaultCollection/" + teamproject, true);
                ReportService.CatalogItem item = null;
                bool exists = false;
                foreach (CatalogItem folder in items)
                {
                    if (folder.Name == "TicketM")
                    {
                        // Console.WriteLine("folder exists");
                        exists = true;
                    }

                }
                //create TicketM folder if it does not exist
                if (!exists)
                {
                    item = rs.CreateFolder("TicketM", "/TfsReports/" + collection + "/" + teamproject, null);
                }
                byte[] definition = null;
                Warning[] warnings = null;
                FileStream stream = File.OpenRead(file);
                definition = new Byte[stream.Length];
                stream.Read(definition, 0, (int)stream.Length);
                stream.Close();
                rs.CreateCatalogItem("Report", reportName, "/TfsReports/" + collection + "/" + teamproject + "/TicketM", true, definition, null, out warnings);
                //rs.CreateCatalogItem("Report", reportName, "/TfsReports/DefaultCollection/" + teamproject + "/TicketM", true, definition, null, out warnings);
                //replace  file contets back to previous one
                str = File.ReadAllText(file);
                str = str.Replace(teamProject, "[SSGS EMS]");
                File.WriteAllText(file, str);


            }
        }
    }
}